// mit Overflow von Timer0 gesteuerter AD-Wandler mit anschl. Berechnungen
// 
// Atmega16 16MHz Takt
// AD-Wandlerkanal ADC0, Bereich 0-5V, 8-bit
// Ausgabe binr am PortC (fr R/2R Netzwerk)
//
// Timer0 soll jedesmal bei Overflow (jede 128us) den AD-Wandler fr eine Einzelwandlung znden
// Anschl. erfolgt eine Berechnung mit dem gewandelten Wert, dabei wird PORTC binr gesetzt
// bis er in nchsten 128us nchsten Wert kriegt, also Sample & Hold.

#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>

#define do_adc 	0

volatile int status_byte=0;
	int e=0;
    int ealt=0;
    int ealtt=0;
	int y=0;
	int yalt=0;
	int x=0;

// AD-Wandler initialisieren
void adc_init(void) {
    // externe Referenzspannung und AD-Wandlerkanal 0 (ADC0) auswhlen
    ADMUX = 0;

    // AD-Wandler einschalten und Prescaler = 16 einstellen
    ADCSRA |= (1<<ADEN) | (1<<ADPS2);
}

// Timer0 initialisieren
void timer0_init(void)
{
	// 16000000 Hz : Teiler 8 : 256 Zhlwerte pro Overflow = 7812,5Hz = 1/7812,5Hz = 128 us
	TCCR0 |= (1<<CS01);
	
	/* Interrupts fr Timer0 aktivieren */	
	TIMSK |= (1<<TOIE0);
}

// AD Wandlung und Berechnung
void rechnung(void)
{
	// Wandlung starten (1<<ADSC), Einzel.Wandl.
    ADCSRA |= (1<<ADSC);

    // Warten bis die AD-Wandlung abgeschloen ist
    while ( !(ADCSRA & (1<<ADIF)) );
	x = ADCH;// 8-bit Wert nach x laden
	
	// Berechnung Anfang
	e = 230 - x;
	y = (yalt + 152*e - 303*ealt + 152*ealtt) / 10000;		//Reglergleichung
	ealtt = ealt;
	ealt = e;
	PORTC=y; //Ausgabe
	yalt = y;
	// Berechnung Ende
}

int main(void) {
	
	DDRC=0xFF;
	timer0_init();
	adc_init();
	sei();
	for(;;){	//Endlosschleife starten
				if(status_byte & (1<<do_adc)){  // wenn do_adc 1 ist folgendes ausfhren
				status_byte &= ~(1<<do_adc);  // erst do_adc lschen
				rechnung(); // AD Wandlung und Berechnung
			}		
	}
}

ISR(TIMER0_OVF_vect)
{
	status_byte |= (1<<do_adc);
}

